@@ -1,21 +1,7 @@
+// SPDX-License-Identifier: LGPL-2.1+
 /*
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301 USA.
- *
- * Copyright 2007 - 2008 Novell, Inc.
- * Copyright 2007 - 2014 Red Hat, Inc.
+ * Copyright (C) 2007 - 2008 Novell, Inc.
+ * Copyright (C) 2007 - 2014 Red Hat, Inc.
  */
 
 #ifndef __NM_CLIENT_H__
@@ -29,6 +15,23 @@
 
 G_BEGIN_DECLS
 
+/**
+ * NMClientInstanceFlags:
+ * @NM_CLIENT_INSTANCE_FLAGS_NONE: special value to indicate no flags.
+ * @NM_CLIENT_INSTANCE_FLAGS_NO_AUTO_FETCH_PERMISSIONS: by default, NMClient
+ *   will fetch the permissions via "GetPermissions" and refetch them when
+ *   "CheckPermissions" signal gets received. By setting this flag, this behavior
+ *   can be disabled. You can toggle this flag to enable and disable automatic
+ *   fetching of the permissions. Watch also nm_client_get_permissions_state()
+ *   to know whether the permissions are up to date.
+ *
+ * Since: 1.24
+ */
+typedef enum { /*< flags >*/
+	NM_CLIENT_INSTANCE_FLAGS_NONE                      = 0,
+	NM_CLIENT_INSTANCE_FLAGS_NO_AUTO_FETCH_PERMISSIONS = 1,
+} NMClientInstanceFlags;
+
 #define NM_TYPE_CLIENT            (nm_client_get_type ())
 #define NM_CLIENT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_CLIENT, NMClient))
 #define NM_CLIENT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), NM_TYPE_CLIENT, NMClientClass))
@@ -36,21 +39,36 @@ G_BEGIN_DECLS
 #define NM_IS_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), NM_TYPE_CLIENT))
 #define NM_CLIENT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_CLIENT, NMClientClass))
 
-#define NM_CLIENT_VERSION "version"
-#define NM_CLIENT_STATE "state"
-#define NM_CLIENT_STARTUP "startup"
-#define NM_CLIENT_NM_RUNNING "nm-running"
+#define NM_CLIENT_VERSION         "version"
+#define NM_CLIENT_STATE           "state"
+#define NM_CLIENT_STARTUP         "startup"
+#define NM_CLIENT_NM_RUNNING      "nm-running"
+#define NM_CLIENT_DBUS_CONNECTION "dbus-connection"
+#define NM_CLIENT_DBUS_NAME_OWNER "dbus-name-owner"
+#define NM_CLIENT_INSTANCE_FLAGS  "instance-flags"
+
+_NM_DEPRECATED_SYNC_WRITABLE_PROPERTY
 #define NM_CLIENT_NETWORKING_ENABLED "networking-enabled"
+
+_NM_DEPRECATED_SYNC_WRITABLE_PROPERTY
 #define NM_CLIENT_WIRELESS_ENABLED "wireless-enabled"
-#define NM_CLIENT_WIRELESS_HARDWARE_ENABLED "wireless-hardware-enabled"
+_NM_DEPRECATED_SYNC_WRITABLE_PROPERTY
 #define NM_CLIENT_WWAN_ENABLED "wwan-enabled"
-#define NM_CLIENT_WWAN_HARDWARE_ENABLED "wwan-hardware-enabled"
+_NM_DEPRECATED_SYNC_WRITABLE_PROPERTY
 #define NM_CLIENT_WIMAX_ENABLED "wimax-enabled"
+
+#define NM_CLIENT_WIRELESS_HARDWARE_ENABLED "wireless-hardware-enabled"
+#define NM_CLIENT_WWAN_HARDWARE_ENABLED "wwan-hardware-enabled"
 #define NM_CLIENT_WIMAX_HARDWARE_ENABLED "wimax-hardware-enabled"
+
 #define NM_CLIENT_ACTIVE_CONNECTIONS "active-connections"
 #define NM_CLIENT_CONNECTIVITY "connectivity"
+#define NM_CLIENT_CONNECTIVITY_CHECK_URI "connectivity-check-uri"
 #define NM_CLIENT_CONNECTIVITY_CHECK_AVAILABLE "connectivity-check-available"
+
+_NM_DEPRECATED_SYNC_WRITABLE_PROPERTY
 #define NM_CLIENT_CONNECTIVITY_CHECK_ENABLED "connectivity-check-enabled"
+
 #define NM_CLIENT_PRIMARY_CONNECTION "primary-connection"
 #define NM_CLIENT_ACTIVATING_CONNECTION "activating-connection"
 #define NM_CLIENT_DEVICES "devices"
@@ -62,6 +80,9 @@ G_BEGIN_DECLS
 #define NM_CLIENT_DNS_MODE "dns-mode"
 #define NM_CLIENT_DNS_RC_MANAGER "dns-rc-manager"
 #define NM_CLIENT_DNS_CONFIGURATION "dns-configuration"
+#define NM_CLIENT_CHECKPOINTS "checkpoints"
+#define NM_CLIENT_CAPABILITIES "capabilities"
+#define NM_CLIENT_PERMISSIONS_STATE "permissions-state"
 
 #define NM_CLIENT_DEVICE_ADDED "device-added"
 #define NM_CLIENT_DEVICE_REMOVED "device-removed"
@@ -73,87 +94,6 @@ G_BEGIN_DECLS
 #define NM_CLIENT_ACTIVE_CONNECTION_ADDED "active-connection-added"
 #define NM_CLIENT_ACTIVE_CONNECTION_REMOVED "active-connection-removed"
 
-/**
- * NMClientPermission:
- * @NM_CLIENT_PERMISSION_NONE: unknown or no permission
- * @NM_CLIENT_PERMISSION_ENABLE_DISABLE_NETWORK: controls whether networking
- *  can be globally enabled or disabled
- * @NM_CLIENT_PERMISSION_ENABLE_DISABLE_WIFI: controls whether Wi-Fi can be
- *  globally enabled or disabled
- * @NM_CLIENT_PERMISSION_ENABLE_DISABLE_WWAN: controls whether WWAN (3G) can be
- *  globally enabled or disabled
- * @NM_CLIENT_PERMISSION_ENABLE_DISABLE_WIMAX: controls whether WiMAX can be
- *  globally enabled or disabled
- * @NM_CLIENT_PERMISSION_SLEEP_WAKE: controls whether the client can ask
- *  NetworkManager to sleep and wake
- * @NM_CLIENT_PERMISSION_NETWORK_CONTROL: controls whether networking connections
- *  can be started, stopped, and changed
- * @NM_CLIENT_PERMISSION_WIFI_SHARE_PROTECTED: controls whether a password
- *  protected Wi-Fi hotspot can be created
- * @NM_CLIENT_PERMISSION_WIFI_SHARE_OPEN: controls whether an open Wi-Fi hotspot
- *  can be created
- * @NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM: controls whether connections
- *  that are available to all users can be modified
- * @NM_CLIENT_PERMISSION_SETTINGS_MODIFY_OWN: controls whether connections
- *  owned by the current user can be modified
- * @NM_CLIENT_PERMISSION_SETTINGS_MODIFY_HOSTNAME: controls whether the
- *  persistent hostname can be changed
- * @NM_CLIENT_PERMISSION_SETTINGS_MODIFY_GLOBAL_DNS: modify persistent global
- *  DNS configuration
- * @NM_CLIENT_PERMISSION_RELOAD: controls access to Reload.
- * @NM_CLIENT_PERMISSION_CHECKPOINT_ROLLBACK: permission to create checkpoints.
- * @NM_CLIENT_PERMISSION_ENABLE_DISABLE_STATISTICS: controls whether device
- *  statistics can be globally enabled or disabled
- * @NM_CLIENT_PERMISSION_ENABLE_DISABLE_CONNECTIVITY_CHECK: controls whether
- *  connectivity check can be enabled or disabled
- * @NM_CLIENT_PERMISSION_WIFI_SCAN: controls whether wifi scans can be performed
- * @NM_CLIENT_PERMISSION_LAST: a reserved boundary value
- *
- * #NMClientPermission values indicate various permissions that NetworkManager
- * clients can obtain to perform certain tasks on behalf of the current user.
- **/
-typedef enum {
-	NM_CLIENT_PERMISSION_NONE = 0,
-	NM_CLIENT_PERMISSION_ENABLE_DISABLE_NETWORK = 1,
-	NM_CLIENT_PERMISSION_ENABLE_DISABLE_WIFI = 2,
-	NM_CLIENT_PERMISSION_ENABLE_DISABLE_WWAN = 3,
-	NM_CLIENT_PERMISSION_ENABLE_DISABLE_WIMAX = 4,
-	NM_CLIENT_PERMISSION_SLEEP_WAKE = 5,
-	NM_CLIENT_PERMISSION_NETWORK_CONTROL = 6,
-	NM_CLIENT_PERMISSION_WIFI_SHARE_PROTECTED = 7,
-	NM_CLIENT_PERMISSION_WIFI_SHARE_OPEN = 8,
-	NM_CLIENT_PERMISSION_SETTINGS_MODIFY_SYSTEM = 9,
-	NM_CLIENT_PERMISSION_SETTINGS_MODIFY_OWN = 10,
-	NM_CLIENT_PERMISSION_SETTINGS_MODIFY_HOSTNAME = 11,
-	NM_CLIENT_PERMISSION_SETTINGS_MODIFY_GLOBAL_DNS = 12,
-	NM_CLIENT_PERMISSION_RELOAD = 13,
-	NM_CLIENT_PERMISSION_CHECKPOINT_ROLLBACK = 14,
-	NM_CLIENT_PERMISSION_ENABLE_DISABLE_STATISTICS = 15,
-	NM_CLIENT_PERMISSION_ENABLE_DISABLE_CONNECTIVITY_CHECK = 16,
-	NM_CLIENT_PERMISSION_WIFI_SCAN = 17,
-
-	NM_CLIENT_PERMISSION_LAST = 17,
-} NMClientPermission;
-
-/**
- * NMClientPermissionResult:
- * @NM_CLIENT_PERMISSION_RESULT_UNKNOWN: unknown or no authorization
- * @NM_CLIENT_PERMISSION_RESULT_YES: the permission is available
- * @NM_CLIENT_PERMISSION_RESULT_AUTH: authorization is necessary before the
- *  permission is available
- * @NM_CLIENT_PERMISSION_RESULT_NO: permission to perform the operation is
- *  denied by system policy
- *
- * #NMClientPermissionResult values indicate what authorizations and permissions
- * the user requires to obtain a given #NMClientPermission
- **/
-typedef enum {
-	NM_CLIENT_PERMISSION_RESULT_UNKNOWN = 0,
-	NM_CLIENT_PERMISSION_RESULT_YES,
-	NM_CLIENT_PERMISSION_RESULT_AUTH,
-	NM_CLIENT_PERMISSION_RESULT_NO
-} NMClientPermissionResult;
-
 /**
  * NMClientError:
  * @NM_CLIENT_ERROR_FAILED: unknown or unclassified error
@@ -200,27 +140,7 @@ gboolean            nm_dns_entry_get_vpn (NMDnsEntry *entry);
 /**
  * NMClient:
  */
-struct _NMClient {
-	GObject parent;
-};
-
-typedef struct {
-	GObjectClass parent;
-
-	/* Signals */
-	void (*device_added) (NMClient *client, NMDevice *device);
-	void (*device_removed) (NMClient *client, NMDevice *device);
-	void (*any_device_added) (NMClient *client, NMDevice *device);
-	void (*any_device_removed) (NMClient *client, NMDevice *device);
-	void (*permission_changed) (NMClient *client,
-	                            NMClientPermission permission,
-	                            NMClientPermissionResult result);
-	void (*connection_added)   (NMClient *client, NMRemoteConnection *connection);
-	void (*connection_removed) (NMClient *client, NMRemoteConnection *connection);
-
-	/*< private >*/
-	gpointer padding[6];
-} NMClientClass;
+typedef struct _NMClientClass NMClientClass;
 
 GType nm_client_get_type (void);
 
@@ -233,26 +153,66 @@ void      nm_client_new_async  (GCancellable         *cancellable,
 NMClient *nm_client_new_finish (GAsyncResult         *result,
                                 GError              **error);
 
+
+NM_AVAILABLE_IN_1_24
+NMClientInstanceFlags nm_client_get_instance_flags (NMClient *self);
+
+NM_AVAILABLE_IN_1_22
+GDBusConnection *nm_client_get_dbus_connection (NMClient *client);
+
+NM_AVAILABLE_IN_1_22
+GMainContext *nm_client_get_main_context (NMClient *self);
+
+NM_AVAILABLE_IN_1_22
+GObject *nm_client_get_context_busy_watcher (NMClient *self);
+
+NM_AVAILABLE_IN_1_22
+const char *nm_client_get_dbus_name_owner (NMClient *client);
+
 const char *nm_client_get_version    (NMClient *client);
 NMState     nm_client_get_state      (NMClient *client);
 gboolean    nm_client_get_startup    (NMClient *client);
 gboolean    nm_client_get_nm_running (NMClient *client);
 
+NMObject *nm_client_get_object_by_path (NMClient *client,
+                                        const char *dbus_path);
+
+NM_AVAILABLE_IN_1_22
+NMMetered   nm_client_get_metered (NMClient *client);
+
 gboolean nm_client_networking_get_enabled (NMClient *client);
+
+NM_AVAILABLE_IN_1_24
+const guint32 *nm_client_get_capabilities (NMClient *client,
+                                           gsize *length);
+
+_NM_DEPRECATED_SYNC_METHOD
 gboolean nm_client_networking_set_enabled (NMClient *client,
                                            gboolean enabled,
                                            GError **error);
 
 gboolean nm_client_wireless_get_enabled (NMClient *client);
+
+_NM_DEPRECATED_SYNC_METHOD
 void     nm_client_wireless_set_enabled (NMClient *client, gboolean enabled);
+
 gboolean nm_client_wireless_hardware_get_enabled (NMClient *client);
 
 gboolean nm_client_wwan_get_enabled (NMClient *client);
+
+_NM_DEPRECATED_SYNC_METHOD
 void     nm_client_wwan_set_enabled (NMClient *client, gboolean enabled);
+
 gboolean nm_client_wwan_hardware_get_enabled (NMClient *client);
 
+NM_DEPRECATED_IN_1_22
 gboolean nm_client_wimax_get_enabled (NMClient *client);
+
+NM_DEPRECATED_IN_1_22
+_NM_DEPRECATED_SYNC_METHOD
 void     nm_client_wimax_set_enabled (NMClient *client, gboolean enabled);
+
+NM_DEPRECATED_IN_1_22
 gboolean nm_client_wimax_hardware_get_enabled (NMClient *client);
 
 NM_AVAILABLE_IN_1_10
@@ -262,16 +222,20 @@ NM_AVAILABLE_IN_1_10
 gboolean nm_client_connectivity_check_get_enabled (NMClient *client);
 
 NM_AVAILABLE_IN_1_10
+_NM_DEPRECATED_SYNC_METHOD
 void     nm_client_connectivity_check_set_enabled (NMClient *client,
                                                    gboolean enabled);
 
 NM_AVAILABLE_IN_1_20
 const char *nm_client_connectivity_check_get_uri (NMClient *client);
 
+_NM_DEPRECATED_SYNC_METHOD
 gboolean nm_client_get_logging (NMClient *client,
                                 char **level,
                                 char **domains,
                                 GError **error);
+
+_NM_DEPRECATED_SYNC_METHOD
 gboolean nm_client_set_logging (NMClient *client,
                                 const char *level,
                                 const char *domains,
@@ -280,11 +244,17 @@ gboolean nm_client_set_logging (NMClient *client,
 NMClientPermissionResult nm_client_get_permission_result (NMClient *client,
                                                           NMClientPermission permission);
 
+NM_AVAILABLE_IN_1_24
+NMTernary nm_client_get_permissions_state (NMClient *self);
+
 NMConnectivityState nm_client_get_connectivity          (NMClient *client);
 
+_NM_DEPRECATED_SYNC_METHOD
+NM_DEPRECATED_IN_1_22
 NMConnectivityState nm_client_check_connectivity        (NMClient *client,
                                                          GCancellable *cancellable,
                                                          GError **error);
+
 void                nm_client_check_connectivity_async  (NMClient *client,
                                                          GCancellable *cancellable,
                                                          GAsyncReadyCallback callback,
@@ -293,10 +263,12 @@ NMConnectivityState nm_client_check_connectivity_finish (NMClient *client,
                                                          GAsyncResult *result,
                                                          GError **error);
 
+_NM_DEPRECATED_SYNC_METHOD
 gboolean nm_client_save_hostname        (NMClient *client,
                                          const char *hostname,
                                          GCancellable *cancellable,
                                          GError **error);
+
 void     nm_client_save_hostname_async  (NMClient *client,
                                          const char *hostname,
                                          GCancellable *cancellable,
@@ -358,10 +330,12 @@ NMActiveConnection *nm_client_add_and_activate_connection2_finish (NMClient *cli
                                                                    GVariant **out_result,
                                                                    GError **error);
 
+_NM_DEPRECATED_SYNC_METHOD
 gboolean nm_client_deactivate_connection        (NMClient *client,
                                                  NMActiveConnection *active,
                                                  GCancellable *cancellable,
                                                  GError **error);
+
 void     nm_client_deactivate_connection_async  (NMClient *client,
                                                  NMActiveConnection *active,
                                                  GCancellable *cancellable,
@@ -405,11 +379,13 @@ NMRemoteConnection *nm_client_add_connection2_finish (NMClient *client,
                                                       GVariant **out_result,
                                                       GError **error);
 
+_NM_DEPRECATED_SYNC_METHOD
 gboolean nm_client_load_connections        (NMClient *client,
                                             char **filenames,
                                             char ***failures,
                                             GCancellable *cancellable,
                                             GError **error);
+
 void     nm_client_load_connections_async  (NMClient *client,
                                             char **filenames,
                                             GCancellable *cancellable,
@@ -420,9 +396,11 @@ gboolean nm_client_load_connections_finish (NMClient *client,
                                             GAsyncResult *result,
                                             GError **error);
 
+_NM_DEPRECATED_SYNC_METHOD
 gboolean nm_client_reload_connections        (NMClient *client,
                                               GCancellable *cancellable,
                                               GError **error);
+
 void     nm_client_reload_connections_async  (NMClient *client,
                                               GCancellable *cancellable,
                                               GAsyncReadyCallback callback,
@@ -489,6 +467,52 @@ gboolean nm_client_checkpoint_adjust_rollback_timeout_finish (NMClient *client,
                                                               GAsyncResult *result,
                                                               GError **error);
 
+NM_AVAILABLE_IN_1_22
+void nm_client_reload (NMClient *client,
+                       NMManagerReloadFlags flags,
+                       GCancellable *cancellable,
+                       GAsyncReadyCallback callback,
+                       gpointer user_data);
+NM_AVAILABLE_IN_1_22
+gboolean nm_client_reload_finish (NMClient *client,
+                                  GAsyncResult *result,
+                                  GError **error);
+
+/*****************************************************************************/
+
+NM_AVAILABLE_IN_1_24
+void nm_client_dbus_call (NMClient *client,
+                          const char *object_path,
+                          const char *interface_name,
+                          const char *method_name,
+                          GVariant *parameters,
+                          const GVariantType *reply_type,
+                          int timeout_msec,
+                          GCancellable *cancellable,
+                          GAsyncReadyCallback callback,
+                          gpointer user_data);
+
+NM_AVAILABLE_IN_1_24
+GVariant *nm_client_dbus_call_finish (NMClient *client,
+                                      GAsyncResult *result,
+                                      GError **error);
+
+NM_AVAILABLE_IN_1_24
+void nm_client_dbus_set_property (NMClient *client,
+                                  const char *object_path,
+                                  const char *interface_name,
+                                  const char *property_name,
+                                  GVariant *value,
+                                  int timeout_msec,
+                                  GCancellable *cancellable,
+                                  GAsyncReadyCallback callback,
+                                  gpointer user_data);
+
+NM_AVAILABLE_IN_1_24
+gboolean nm_client_dbus_set_property_finish (NMClient *client,
+                                             GAsyncResult *result,
+                                             GError **error);
+
 G_END_DECLS
 
 #endif /* __NM_CLIENT_H__ */
